﻿using System;

namespace VersionTask
{
    public class Version : VersionInfo, IComparable<Version>
    {
        public Version(ulong major, ulong minor, ulong patch) 
            : base(major, minor, patch)
        { }

        public int CompareTo(Version other)
        {
            var thisData = new[] { Major, Minor, Patch };
            var otherData = new[] { other.Major, other.Minor, other.Patch };

            for (var i = 0; i < 3; i++)
            {
                var res = thisData[i].CompareTo(otherData[i]);
                if (res != 0)
                    return res;
            }
            
            return 0;
        }
        
        public static bool operator <(Version version1, Version version2)
        {
            return version1.CompareTo(version2) < 0;
        }
        
        public static bool operator >(Version version1, Version version2)
        {
            return version1.CompareTo(version2) > 0;
        }
        
        public static bool operator ==(Version version1, Version version2)
        {
            return version1?.CompareTo(version2) == 0;
        }
        
        public static bool operator !=(Version version1, Version version2)
        {
             return version1?.CompareTo(version2) != 0; 
        }
        
        public static Version operator +(Version version1, Version version2)
        {
            return new Version(version1.Major + version2.Major, version1.Minor + version2.Minor, version1.Patch + version2.Patch);
        }
        
        public static Version operator -(Version version1, Version version2)
        {
            
            return new Version(GetZeroIfSubtractionMinus(version1.Major, version2.Major), GetZeroIfSubtractionMinus(version1.Minor, version2.Minor),
                GetZeroIfSubtractionMinus(version1.Patch, version2.Patch));
        }

        private static ulong GetZeroIfSubtractionMinus(ulong first, ulong second) =>
            first < second
                ? 0
                : first - second;
    }
}